/**
  ******************************************************************************
  * @file    sh32f9xx_sc0_amoc.h
  * @author  sinowealth
  * @version V1.1.0
  * @date    2021-11-11  
  * @brief   This file contains all the functions prototypes for the TIMER firmware
  *          library.
  ******************************************************************************
  * @attention
  *
  * SINOWEALTH IS SUPPLYING THIS SOFTWARE FOR USE EXCLUSIVELY SH_ON SINOWEALTH'S 
  * MICROCONTROLLER PRODUCTS. IT IS PROTECTED UNDER APPLICABLE COPYRIGHT LAWS. 
  * THIS SOFTWARE IS FOR GUIDANCE IN ORDER TO SAVE TIME. AS A RESULT, SINOWEALTH 
  * SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES 
  * WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT OF SUCH FIRMWARE AND/OR
  * THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION CONTAINED HEREIN IN 
  * CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2020 Sinowealth</center></h2>
  *
  *
  ******************************************************************************
  */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __SH32F9XX_SC0_AMOC_H
#define __SH32F9XX_SC0_AMOC_H

#ifdef __cplusplus
extern "C"
{
#endif

/* Includes ------------------------------------------------------------------*/
#include "sh32f9xx_sc0_libcfg.h"

/** @addtogroup SH32F9xx_sc0_StdLib_Driver
  * @{
  */

/** @addtogroup AMOC_MODULE
  * @{
  */

/* Exported types ------------------------------------------------------------*/
 
 
/**
  * @brief define the AMOC amplifier
  * @arg   AMOC_AMPL0:  amplifier 0
  * @arg   AMOC_AMPL1:  amplifier 1
  * @arg   AMOC_AMPL2:  amplifier 2
  */    
typedef enum 
{ 
    AMOC_AMPL0 = 0, 
    AMOC_AMPL1 = 1,
    AMOC_AMPL2 = 2, 
}CMP_AMPL_TypeDef; 
 
#define IS_CMP_AMPLX_TypeDef(CMPAMPL)   ((CMPAMPL == AMOC_AMPL0)  \
                                        || (CMPAMPL == AMOC_AMPL1) \
                                        || (CMPAMPL == AMOC_AMPL2))

/**
  * @brief define the AMOC comparator control
  * @arg   AMOC_CMP0_CTR:  AMOC comparator 0 control
  * @arg   AMOC_CMP1_CTR:  AMOC comparator 1 control
  * @arg   AMOC_CMP2_CTR:  AMOC comparator 2 control
  */    
typedef enum 
{ 
    AMOC_CMP0_CTR = 0, 
    AMOC_CMP1_CTR = 1,
    AMOC_CMP2_CTR = 2, 
}CMP_CTR_TypeDef; 
 
#define IS_CMP_CTR_TypeDef(CMPCTR)   ((CMPCTR == AMOC_CMP0_CTR)  \
                                     || (CMPCTR == AMOC_CMP1_CTR) \
                                     || (CMPCTR == AMOC_CMP2_CTR))

/**
  * @brief define the reference voltage source
  * @arg   AVDD_REF:    AVDD
  * @arg   VREF_REF:  Internal Vref
  */    
typedef enum 
{ 
    AVDD_REF = 0UL, 
    VREF_REF = !AVDD_REF, 
}CMP_VCMP_TypeDef;

#define IS_CMP_VCMP_TypeDef(CMPVCMP)   ((CMPVCMP == AVDD_REF)  \
                                       || (CMPVCMP == VREF_REF))

/**
  * @brief define the inverter reference voltage selection 
  * @arg   INVER_REFVOL_8:  Vcmp      
  * @arg   INVER_REFVOL_1:  1 x Vcmp/8
  * @arg   INVER_REFVOL_2:  2 x Vcmp/8
  * @arg   INVER_REFVOL_3:  3 x Vcmp/8
  * @arg   INVER_REFVOL_4:  4 x Vcmp/8
  * @arg   INVER_REFVOL_5:  5 x Vcmp/8
  * @arg   INVER_REFVOL_6:  6 x Vcmp/8
  * @arg   INVER_REFVOL_7:  7 x Vcmp/8
  */    
typedef enum 
{ 
    INVER_REFVOL_8 = 0,   
    INVER_REFVOL_1 = 1,   
    INVER_REFVOL_2 = 2,   
    INVER_REFVOL_3 = 3,   
    INVER_REFVOL_4 = 4,   
    INVER_REFVOL_5 = 5,   
    INVER_REFVOL_6 = 6,   
    INVER_REFVOL_7 = 7, 
}CMP_REFVOL_SEL_TypeDef;

#define IS_CMP_REFVOL_SEL_TypeDef(VOLT)   ((VOLT == INVER_REFVOL_8)  \
                                          || (VOLT == INVER_REFVOL_1)\
                                          || (VOLT == INVER_REFVOL_2)\
                                          || (VOLT == INVER_REFVOL_3)\
                                          || (VOLT == INVER_REFVOL_4)\
                                          || (VOLT == INVER_REFVOL_5)\
                                          || (VOLT == INVER_REFVOL_6)\
                                          || (VOLT == INVER_REFVOL_7))
/**
  * @brief define the Comparator 0 interrupt mode selection  
  * @arg   INT_MODE_NO  :  no interrupt      
  * @arg   INT_MODE_FALL:  fall edge
  * @arg   INT_MODE_RISE:  rise edge
  * @arg   INT_MODE_FLRI:  double edge
  */    
typedef enum 
{ 
    INT_MODE_NO = 0,   
    INT_MODE_FALL = 1,   
    INT_MODE_RISE = 2,   
    INT_MODE_FLRI = 3, 
}CMP_INTMOD_SEL_TypeDef;

#define IS_CMP_INTMOD_SEL_TypeDef(MODE)   ((MODE == INT_MODE_NO)     \
                                          || (MODE == INT_MODE_FALL) \
                                          || (MODE == INT_MODE_RISE) \
                                          || (MODE == INT_MODE_FLRI))

/**
  * @brief Comparator x Schmitt voltage selection bit  
  * @arg   SMIT_VOL_NO  :  no Schmitt window features     
  * @arg   SMIT_VOL_10MV:  Schmitt window is 10mV
  * @arg   SMIT_VOL_20MV:  Schmitt window is 20mV      
  * @arg   SMIT_VOL_50MV:  Schmitt window is 50mV
  */    
typedef enum 
{ 
    SMIT_VOL_NO   = 0,        
    SMIT_VOL_10MV = 1,     
    SMIT_VOL_20MV = 2,
    SMIT_VOL_50MV = 3,
}CMP_SMIT_VOLSEL_TypeDef;

#define IS_CMP_SMIT_VOLSEL_TypeDef(SMITVOL)    ((SMITVOL == SMIT_VOL_NO )   \
                                               || (SMITVOL == SMIT_VOL_10MV)\
                                               || (SMITVOL == SMIT_VOL_20MV)\
                                               || (SMITVOL == SMIT_VOL_50MV))

/**
  * @brief Comparator x inverting input terminal selection bit  
  * @arg   CMP_CN0:  Select C0N as the inverting input terminal of comparator X     
  * @arg   CMP_VRS:  Select the internal reference CMP0VRS as the inverting input of comparator X
  */    
typedef enum 
{ 
    CMP_CN0   = 0,        
    CMP_VRS   = 1,     
}CMP_INVERT_INSEL_TypeDef;

#define IS_CMP_INVERT_INSEL_TypeDef(INVIN)    ((INVIN == CMP_CN0 )   \
                                              || (INVIN == CMP_VRS))


/**
  * @brief Comparator 0 non-inverting input terminal selection bit 
  * @arg   CMP0_C0P0:  Seleccione C0P0 as terminal  no inversora of comparador 0    
  * @arg   CMP0_C0P1:  Seleccione C0P1 as terminal  no inversora of comparador 0
  */    
typedef enum 
{ 
    CMP0_C0P0   = 0,        
    CMP0_C0P1   = 1,
}CMP0_FRWRD_INSEL_TypeDef;

#define IS_CMP0_FRWRD_INSEL_TypeDef(FRWINSEL0)    ((FRWINSEL0 == CMP0_C0P0 )   \
                                                  || (FRWINSEL0 == CMP0_C0P1))

/**
  * @brief Comparator 1 non-inverting input terminal selection bit 
  * @arg   CMP1_C1P0  :  Seleccione C1P0 as terminal  no inversora of comparador 1    
  * @arg   CMP1_C1P1  :  Seleccione C1P1 as terminal  no inversora of comparador 1
  * @arg   CMP1_C1P2  :  Seleccione C1P2 as terminal  no inversora of comparador 1    
  * @arg   CMP1_OP0OUT:  Seleccione CMP_OP0OUT as terminal  no inversora of comparador 1
  * @arg   CMP1_OP1OUT:  Seleccione CMP_OP1OUT as terminal  no inversora of comparador 1    
  * @arg   CMP1_OP2OUT:  Seleccione CMP_OP2OUT as terminal  no inversora of comparador 1
  */    
typedef enum 
{ 
    CMP1_C1P0     = 0,        
    CMP1_C1P1     = 1,
    CMP1_C1P2     = 2,
    CMP1_OP0OUT   = 3,
    CMP1_OP1OUT   = 4,
    CMP1_OP2OUT   = 5, 
}CMP1_FRWRD_INSEL_TypeDef;

#define IS_CMP1_FRWRD_INSEL_TypeDef(FRWINSEL1)    ((FRWINSEL1 == CMP1_C1P0 )\
                                                  || (FRWINSEL1 == CMP1_C1P1)\
                                                  || (FRWINSEL1 == CMP1_C1P2) \
                                                  || (FRWINSEL1 == CMP1_OP0OUT)\
                                                  || (FRWINSEL1 == CMP1_OP1OUT)\
                                                  || (FRWINSEL1 == CMP1_OP2OUT))

/**
  * @brief Comparator 2 non-inverting input terminal selection bit 
  * @arg   CMP2_C2P0:  Seleccione C2P0 as terminal  no inversora of comparador 2    
  * @arg   CMP2_C2P1:  Seleccione C2P1 as terminal  no inversora of comparador 2   
  * @arg   CMP2_C2P1:  Seleccione CMP2_C2P1 as terminal  no inversora of comparador 2
  * @arg   CMP2_C2P0:  Seleccione CMP2_C2P0 as terminal  no inversora of comparador 2    
  * @arg   CMP2_C2P1:  Seleccione CMP2_C2P1 as terminal  no inversora of comparador 2
  */    
typedef enum 
{ 
    CMP2_C2P0     = 0,        
    CMP2_C2P1     = 1,
    CMP2_OP0OUT   = 2,
    CMP2_OP1OUT   = 3,
    CMP2_OP2OUT   = 4, 
}CMP2_FRWRD_INSEL_TypeDef;

#define IS_CMP2_FRWRD_INSEL_TypeDef(FRWINSEL2)    ((FRWINSEL2 == CMP2_C2P0 )\
                                                  || (FRWINSEL2 == CMP2_C2P1)  \
                                                  || (FRWINSEL2 == CMP2_OP0OUT)\
                                                  || (FRWINSEL2 == CMP2_OP1OUT)\
                                                  || (FRWINSEL2 == CMP2_OP2OUT))
/**
  * @brief define the Comparator x output signal filter time 
  * @arg   FILT_TIMR_NO  :  no    filter time      
  * @arg   FILT_TIMR_HALF:  0.5us filter time
  * @arg   FILT_TIMR_1   :  1us   filter time
  * @arg   FILT_TIMR_2   :  2us   filter time
  * @arg   FILT_TIMR_4   :  4us   filter time
  * @arg   FILT_TIMR_8   :  8us   filter time
  * @arg   FILT_TIMR_12  :  12us  filter time
  * @arg   FILT_TIMR_16  :  16us  filter time
  */    
typedef enum 
{ 
    FILT_TIMR_NO   = 0,   
    FILT_TIMR_HALF = 1,   
    FILT_TIMR_1    = 2,   
    FILT_TIMR_2    = 3,   
    FILT_TIMR_4    = 4,   
    FILT_TIMR_8    = 5,   
    FILT_TIMR_12   = 6,   
    FILT_TIMR_16   = 7, 
}CMP_FILT_TIMR_TypeDef;

#define IS_CMP_FILT_TIMR_TypeDef(FILTTIM)   ((FILTTIM == FILT_TIMR_NO)  \
                                            || (FILTTIM == FILT_TIMR_HALF)\
                                            || (FILTTIM == FILT_TIMR_1   )\
                                            || (FILTTIM == FILT_TIMR_2   )\
                                            || (FILTTIM == FILT_TIMR_4   )\
                                            || (FILTTIM == FILT_TIMR_8   )\
                                            || (FILTTIM == FILT_TIMR_12  )\
                                            || (FILTTIM == FILT_TIMR_16  ))
/** @defgroup AMOC_Exported_Types
  * @{
  */
typedef struct
{
    CMP_CTR_TypeDef          AMOC_CMPX_CTR;   /*!< Comparator x inverter terminal reference voltage source Vcmp selector bit, 
                                      This parameter can be a value of @ref CMP_VCMP_TypeDef */
    CMP_VCMP_TypeDef         AMOC_CMP_Vcmp;   /*!< Comparator x inverter terminal reference voltage source Vcmp selector bit, 
                                      This parameter can be a value of @ref CMP_VCMP_TypeDef */
    CMP_REFVOL_SEL_TypeDef   AMOC_CMP_VOL_SEL;   /*!< Comparator x inverting terminal reference voltage selection bit,  
                                       This parameter can be a value of @ref CMP_REFVOL_SEL_TypeDef */
    FunctionalState          AMOC_CMP_DMAEN;   /*!< Comparator X triggers DMA enable bit, 
                                      This parameter can be a value of @ref FunctionalState */
    CMP_INTMOD_SEL_TypeDef   AMOC_CMP_INT_MODSEL;   /*!< Comparator x interrupt mode selection bit, 
                                      This parameter can be a value of @ref CMP_INTMOD_SEL_TypeDef */
    FunctionalState          AMOC_CMP_OUTEN;   /*!< Comparator X I/O output port enable, 
                                      This parameter can be a value of @ref FunctionalState */
    CMP_SMIT_VOLSEL_TypeDef  AMOC_CMP_SMIT_VOLSEL;   /*!< Comparator x Schmitt voltage selection bit, 
                                     This parameter can be a value of @ref CMP_SMIT_VOLSEL_TypeDef */
    CmdState                 AMOC_CMP_CMPONOROFF;   /*!< Comparator X enable control bit, 
                                     This parameter can be a value of @ref CmdState */
    CMP_INVERT_INSEL_TypeDef AMOC_CMP_INVERT_INSEL;   /*!< Comparator x inverting input terminal selection bit, 
                                     This parameter can be a value of @ref CMP_INVERT_INSEL_TypeDef */
    CMP_FILT_TIMR_TypeDef    AMOC_CMP_OUT_FILTETIM;   /*!< Comparator x output signal filter time, 
                                     This parameter can be a value of @ref CMP_FILT_TIMR_TypeDef */ 
       
} AMOC_CMP_Init_TypeDef;

/**
  * @brief Comparator x interrupt flag clear  
  * @arg   CMP0_INT_CLR:  Comparator 0 output rising edge interrupt flag clear bit    
  * @arg   CMP1_INT_CLR:  Comparator 1 output rising edge interrupt flag clear bit
  * @arg   CMP2_INT_CLR:  Comparator 2 output rising edge interrupt flag clear bit   
  */    
typedef enum 
{ 
    CMP0_INT_CLR = AMOC_CMPINTF_C0IFC_Msk,        
    CMP1_INT_CLR = AMOC_CMPINTF_C1IFC_Msk,     
    CMP2_INT_CLR = AMOC_CMPINTF_C2IFC_Msk,
}CMP_INTCLR_TypeDef;

#define IS_CMP_INTCLR_TypeDef(INTCLR)      ((INTCLR == CMP0_INT_CLR)  \
                                           || (INTCLR == CMP1_INT_CLR)\
                                           || (INTCLR == CMP2_INT_CLR))

/**
  * @brief Built-in voltage output selection bit  
  * @arg   AMOC_VREF_VOL0:  Choose 2.5V as the built-in VREF     
  * @arg   AMOC_VREF_VOL1:  Choose 3.3V as the built-in VREF  
  */    
typedef enum 
{ 
    AMOC_VREF_VOL0 = AMOC_AVREFCON_VREFSEL_Msk,        
    AMOC_VREF_VOL1 = ~((uint8_t)(AMOC_OPCON_OP1EN_Msk)),   
}CMP_BUILDIN_VREFVOL_TypeDef;

#define IS_CMP_BUILDIN_VREFVOL_TypeDef(VREFVOL)      ((VREFVOL == AMOC_VREF_VOL0)  \
                                                     || (VREFVOL == AMOC_VREF_VOL1))

/**
  * @}
  */

/**
  * @}
  */
 
/* Exported macro ------------------------------------------------------------*/
/* Exported functions --------------------------------------------------------*/


void AMOC_StructInit(AMOC_CMP_Init_TypeDef *AMOC_InitStruct);
void AMOC_CTR_Init(AMOC_TypeDef *AMOCx, AMOC_CMP_Init_TypeDef *AMOC_InitStruct);
void AMOC_CMP_OUTPUTEN(CMP_CTR_TypeDef AMOC_CMPAX_CTR, FunctionalState AMOC_CMPX_OUTEN);
void AMOC_CMP_ONOROFF(CMP_CTR_TypeDef AMOC_CMPAX_CTR, CmdState AMOC_CMPX_ONOROFF);
LevelStatus AMOC_CMP_OUTGET(CMP_CTR_TypeDef AMOC_CMPAX_CTR);
FlagStatus AMOC_CMP_INTIFGET(CMP_CTR_TypeDef AMOC_CMPAX_CTR);
void AMOC_CMP_FLGCLER(CMP_CTR_TypeDef AMOC_CMPAX_CTR, CMP_INTCLR_TypeDef AMOC_CMPX_FLGCLR);
void AMOC_CMP0_POSINPUTSEL(CMP0_FRWRD_INSEL_TypeDef AMOC_CMP0_POSINPUT);
void AMOC_CMP1_POSINPUTSEL(CMP1_FRWRD_INSEL_TypeDef AMOC_CMP1_POSINPUT);
void AMOC_CMP2_POSINPUTSEL(CMP2_FRWRD_INSEL_TypeDef AMOC_CMP2_POSINPUT);
void AMOC_AMPL_DISOREN(CMP_AMPL_TypeDef AMOC_AMPLX, FunctionalState AMOC_AMPLX_DISOREN);
void AMOC_VREF_BUILTIN(CMP_BUILDIN_VREFVOL_TypeDef AMOC_VREF_BUIN);
void AMOC_VOLIN_ONOROFF(CmdState AMOC_VOLBUIN_ONOROFF);


#ifdef __cplusplus
}
#endif

#endif /*__SH32F9XX_SC0_AMOC_H */

/**
  * @}
  */

/**
  * @}
  */

/************************ (C) COPYRIGHT Sinowealth *****END OF FILE****/
