/**
  ******************************************************************************
  * @file    sh32f9xx_sc0_rcc.c
  * @author  sinowealth
  * @version V1.1.0
  * @date    2021-05-21  
  * @brief   This file provides reset and clock module's APIs
  *         
  *  @verbatim
  *
  *          ===================================================================
  *                                   How to use this driver
  *          ===================================================================
  *          
  *
  *  @endverbatim
  *
  ******************************************************************************
  * @attention
  *
  * SINOWEALTH IS SUPPLYING THIS SOFTWARE FOR USE EXCLUSIVELY SH_ON SINOWEALTH'S 
  * MICROCONTROLLER PRODUCTS. IT IS PROTECTED UNDER APPLICABLE COPYRIGHT LAWS. 
  * THIS SOFTWARE IS FOR GUIDANCE IN ORDER TO SAVE TIME. AS A RESULT, SINOWEALTH 
  * SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES 
  * WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT OF SUCH FIRMWARE AND/OR
  * THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION CONTAINED HEREIN IN 
  * CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2021 Sinowealth</center></h2>
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "sh32f9xx_sc0_rcc.h"
#include "system_sh32f9xx_sc0.h"

/**
  ******************************************************************************
  * @function  RCC_GetClocksFreq
  * @brief     Returns the frequencies of different on chip clocks.
  * @version   V1.1.0
  * @date      19-MAY-2021
  * @param     Clocks: pointer to a RCC_Clocks_TypeDef structure which will hold
  *            the clocks frequencies.
  * @retval    None
  ******************************************************************************
  */
void RCC_GetClocksFreq(RCC_Clocks_TypeDef *Clocks)
{
    uint8_t tmp = 0;
    uint8_t pllmul_k = 0;
    uint8_t pllmul_f = 0;
    uint8_t pllxtpre = 0;
    uint8_t pllsrc = 0;
    uint8_t hpre_div = 0;
    uint32_t sysClk = 0;

    /* Get SYSCLK source -------------------------------------------------------*/
    tmp = RCC->CR.BIT.SWS;
    hpre_div = RCC->CFGR.BIT.HPRE;

    switch (tmp)
    {
        case RCC_SYS_SRC_LSI: /* LSI used as system clock */
        sysClk = LSI_VALUE;
        break;
        case RCC_SYS_SRC_HSI: /* HSI used as system clock*/
        sysClk = HSI_VALUE;
        break;
        case RCC_SYS_SRC_PLL: /* PLL used as system clock */
        pllsrc = RCC->CFGR.BIT.PLLSRC;
        pllmul_f = RCC->CFGR.BIT.PLLF;
        pllmul_k = RCC->CFGR.BIT.PLLK + 1;
        pllxtpre = RCC->CFGR.BIT.PLLXTPRE + 1;
        if (pllsrc) //HSE as PLL Clock input
        {
            sysClk = HSE_VALUE / pllxtpre * pllmul_f / pllmul_k/2;
        }
        else //HSI as PLL Clock input
        {
            sysClk = HSI_VALUE / pllxtpre * pllmul_f / pllmul_k/2;
        }
        break;
        case RCC_SYS_SRC_HSE: /* HSE used as system clock */
        sysClk = HSE_VALUE;
        break;
    default: /* LSI used as system clock */
        sysClk = LSI_VALUE;
        break;
    }

    Clocks->sysFreq = sysClk;

    //Calculate HCLK
    switch (hpre_div)
    {
    case 0:
        Clocks->hclkFreq = sysClk;
        break;
    case 1:
        Clocks->hclkFreq = sysClk / 2;
        break;
    case 2:
        Clocks->hclkFreq = sysClk / 4;
        break;
    case 3:
        Clocks->hclkFreq = sysClk / 8;
        break;
    case 4:
        Clocks->hclkFreq = sysClk / 16;
        break;
    case 5:
        Clocks->hclkFreq = sysClk / 32;
        break;
    default:
        Clocks->hclkFreq = sysClk / 32;
        break;
    }

    /*Calculate PCLK0 PCLK1*/
    Clocks->pclk0Freq = Clocks->hclkFreq >> (RCC->CFGR.BIT.PPRE0 > 4 ? 4 : (RCC->CFGR.BIT.PPRE0));
    Clocks->pclk1Freq = Clocks->hclkFreq >> (RCC->CFGR.BIT.PPRE1 > 4 ? 4 : (RCC->CFGR.BIT.PPRE1));
}

/**
  * @brief     open or close AHB modules' clock gate
  * @param  AHBModules:  AHB module bits @ref RCC_AHB_Type
  *   This parameter can be any combination of following values:
  *     @arg @b  RCC_AHB_GPIO    : for GPIO    Module 
  *     @arg @b  RCC_AHB_SYSCFG  : for SYSCFG  Module
  *     @arg @b  RCC_AHB_DMA     : for DMA     Module
  *     @arg @b  RCC_AHB_CRC     : for CRC     Module
  *     @arg @b  RCC_AHB_RAMBIST : for RAMBIST Module
  *     @arg @b  RCC_AHB_ADC0    : for ADC0    Module
  *     @arg @b  RCC_AHB_ADC1    : for ADC1    Module
  *     @arg @b  RCC_AHB_GPT     : for GPT     Module
  *     @arg @b  RCC_AHB_GPT0    : for GPT0    Module
  *     @arg @b  RCC_AHB_GPT1    : for GPT1    Module
  *     @arg @b  RCC_AHB_GPT2    : for GPT2    Module
  *     @arg @b  RCC_AHB_GPT3    : for GPT3    Module
  * @param OnOffState  open or close related clock gate
  *     @arg @b  SH_ON  open related clock gate
  *     @arg @b  SH_OFF  close related clock gate
  * @retval   None
  * */
void RCC_AHBPeriphClockOnOff(RCC_AHB_Type AHBModules, CmdState OnOffState)
{
    assert_param(IS_AHB_MODULES(AHBModules));
    assert_param(IS_CMD_STATE(OnOffState));

    /* Unlock RCC registers */
    RCC_REGS_UNLOCK();

    if (OnOffState == SH_ON)
    {
        RCC->AHBENR.V32 |= AHBModules;
    }
    else
    {
        RCC->AHBENR.V32 &= ~AHBModules;
    }

    /* Lock RCC registers */
    RCC_REGS_LOCK();
}

/**
  * @brief  open or close APB1 modules' clock gate
  * @param  APB1Modules:  APB1 module bits @ref RCC_APB1_Type
  *   This parameter can be any combination of following values:
  *     @arg @b RCC_APB1_EXTI : for EXTI Module
  * @param OnOffState  open or close related clock gate
  *     @arg @b  SH_ON   open related clock gate
  *     @arg @b  SH_OFF  close related clock gate
  * @retval  None
  **/
void RCC_APB1PeriphClockOnOff(RCC_APB1_Type APB1Modules, CmdState OnOffState)
{
    assert_param(IS_APB1_MODULES(APB1Modules));
    assert_param(IS_CMD_STATE(OnOffState));

    /* Unlock APB1_CFG registers */
    APB1_CFG_REGS_UNLOCK();

    if (OnOffState == SH_ON)
    {
        APB1_CFG->APB1ENR.V32 |= APB1Modules;
    }
    else
    {
        APB1_CFG->APB1ENR.V32 &= ~APB1Modules;
    }

    /* Lock APB0_CFG registers */
    APB1_CFG_REGS_LOCK();
}

/**
  * @brief      open or close APB0 modules' clock gate
  * @param  APB0Modules:  APB0 module bits @ref RCC_APB0_Type
  *   This parameter can be any combination of following values:
  *     @arg @b    RCC_APB0_TIM5 : for TIM5  Module
  *     @arg @b    RCC_APB0_TIM6 : for TIM6  Module
  *     @arg @b    RCC_APB0_TIM7 : for TIM7  Module
  *     @arg @b    RCC_APB0_TIM8 : for TIM8  Module
  *     @arg @b    RCC_APB0_UART0: for UART0 Module
  *     @arg @b    RCC_APB0_UART1: for UART1 Module
  *     @arg @b    RCC_APB0_UART2: for UART2 Module
  *     @arg @b    RCC_APB0_UART3: for UART3 Module
  *     @arg @b    RCC_APB0_UART4: for UART4 Module
  *     @arg @b    RCC_APB0_UART5: for UART5 Module
  *     @arg @b    RCC_APB0_SPI0 : for SPI0  Module
  *     @arg @b    RCC_APB0_SPI1 : for SPI1  Module
  *     @arg @b    RCC_APB0_TWI0 : for TWI0  Module
  *     @arg @b    RCC_APB0_TWI1 : for TWI1  Module
  *     @arg @b    RCC_APB0_TK   : for TK    Module
  *     @arg @b    RCC_APB0_WWDT : for WWDT  Module
  *@param OnOffState  open or close related clock gate
  *     @arg @b    SH_ON   open related clock gate
  *     @arg @b    SH_OFF  close related clock gate
  * @retval None
  **/
void RCC_APB0PeriphClockOnOff(RCC_APB0_Type APB0Modules, CmdState OnOffState)
{
    assert_param(IS_APB0_MODULES(APB0Modules));
    assert_param(IS_CMD_STATE(OnOffState));

    /* Unlock APB0_CFG registers */
    APB0_CFG_REGS_UNLOCK();

    if (OnOffState == SH_ON)
    {
        APB0_CFG->APB0ENR.V32 |= APB0Modules;
    }
    else
    {
        APB0_CFG->APB0ENR.V32 &= ~APB0Modules;
    }

    /* Lock APB0_CFG registers */
    APB0_CFG_REGS_LOCK();
}

/**
  * @brief     reset AHB modules
  * @param  AHBModules: AHB module bits @ref RCC_AHB_Type
  *   This parameter can be any combination of following values:
  *     @arg @b  RCC_AHB_GPIO    : for GPIO    Module 
  *     @arg @b  RCC_AHB_SYSCFG  : for SYSCFG  Module
  *     @arg @b  RCC_AHB_DMA     : for DMA     Module
  *     @arg @b  RCC_AHB_CRC     : for CRC     Module
  *     @arg @b  RCC_AHB_RAMBIST : for RAMBIST Module
  *     @arg @b  RCC_AHB_ADC0    : for ADC0    Module
  *     @arg @b  RCC_AHB_ADC1    : for ADC1    Module
  *     @arg @b  RCC_AHB_GPT     : for GPT     Module
  *     @arg @b  RCC_AHB_GPT0    : for GPT0    Module
  *     @arg @b  RCC_AHB_GPT1    : for GPT1    Module
  *     @arg @b  RCC_AHB_GPT2    : for GPT2    Module
  *     @arg @b  RCC_AHB_GPT3    : for GPT3    Module
  * @retval   None
  * */
void RCC_AHBPeriphReset(uint32_t AHBModules)
{
    assert_param(IS_AHB_MODULES(AHBModules));

    /* Unlock RCC registers */
    RCC_REGS_UNLOCK();

    RCC->AHBRSTR.V32 = AHBModules;

    /* Lock RCC registers */
    RCC_REGS_LOCK();
}

/**
  * @brief    reset APB0 modules
  * @param  APB0Modules:  APB0 module bits @ref RCC_APB0_Type
  *   This parameter can be any combination of following values:
  *     @arg @b    RCC_APB0_TIM5 : for TIM5  Module
  *     @arg @b    RCC_APB0_TIM6 : for TIM6  Module
  *     @arg @b    RCC_APB0_TIM7 : for TIM7  Module
  *     @arg @b    RCC_APB0_TIM8 : for TIM8  Module
  *     @arg @b    RCC_APB0_UART0: for UART0 Module
  *     @arg @b    RCC_APB0_UART1: for UART1 Module
  *     @arg @b    RCC_APB0_UART2: for UART2 Module
  *     @arg @b    RCC_APB0_UART3: for UART3 Module
  *     @arg @b    RCC_APB0_UART4: for UART4 Module
  *     @arg @b    RCC_APB0_UART5: for UART5 Module
  *     @arg @b    RCC_APB0_SPI0 : for SPI0  Module
  *     @arg @b    RCC_APB0_SPI1 : for SPI1  Module
  *     @arg @b    RCC_APB0_TWI0 : for TWI0  Module
  *     @arg @b    RCC_APB0_TWI1 : for TWI1  Module
  *     @arg @b    RCC_APB0_TK   : for TK    Module
  *     @arg @b    RCC_APB0_WWDT : for WWDT  Module
  * @retval  None
  **/
void RCC_APB0PeriphReset(uint32_t APB0Modules)
{
    assert_param(IS_APB0_MODULES(APB0Modules));

    /* Unlock APB0_CFG registers */
    APB0_CFG_REGS_UNLOCK();

    APB0_CFG->APB0RSTR.V32 = APB0Modules;

    /* Lock APB0_CFG registers */
    APB0_CFG_REGS_LOCK();
}

/**
  * @brief     reset APB1 modules
  * @param  APB1Modules:  APB1 module bits @ref RCC_APB1_Type
  *   This parameter can be any combination of following values:
        RCC_APB1_EXTI : for EXTI Module
  * @retval None
  **/
void RCC_APB1PeriphReset(uint32_t APB1Modules)
{
    assert_param(IS_APB1_MODULES(APB1Modules));

    /* Unlock APB1_CFG registers */
    APB1_CFG_REGS_UNLOCK();

    APB1_CFG->APB1RSTR.V32 = APB1Modules;

    /* Lock APB1_CFG registers */
    APB1_CFG_REGS_LOCK();
}

/**
  * @brief     Get RCC reset flag
  * @param  ResetFlag:   RCC reset flag mask
  *     @arg @b  RCC_RST_PIN
  *     @arg @b  RCC_RST_LVR
  *     @arg @b  RCC_RST_POWERON
  *     @arg @b  RCC_RST_SOFTWARE
  *     @arg @b  RCC_RST_IWDT
  *     @arg @b  RCC_RST_WWDT
  *     @arg @b  RCC_RST_VCCLVR
  * @retval  FlagStatus RCC reset flag
  *     @arg @b  SH_SET    the flag is setted
  *     @arg @b  RESET  the flag is cleared
  **/
FlagStatus RCC_GetResetFlag(RCC_RESET_Type ResetFlag)
{
    assert_param(IS_RST_MODULES(ResetFlag));

    return ((RCC->RSTSTR.V32 & ResetFlag) ? SH_SET : SH_RESET);
}

/**
  * @brief      clear RCC reset flags
  * @param  ResetFlags:   RCC reset flag clear bits
  *     @arg @b  RCC_RST_PIN
  *     @arg @b  RCC_RST_LVR
  *     @arg @b  RCC_RST_POWERON
  *     @arg @b  RCC_RST_SOFTWARE
  *     @arg @b  RCC_RST_IWDT
  *     @arg @b  RCC_RST_WWDT
  *     @arg @b  RCC_RST_VCCLVR
  * @retval None
  **/
void RCC_ClearResetFlag(RCC_RESET_Type ResetFlags)
{
    assert_param(IS_RST_MODULES(ResetFlags));

    /* Unlock RCC registers */
    RCC_REGS_UNLOCK();

    RCC->RSTCLR.V32 = ResetFlags;

    /* Lock RCC registers */
    RCC_REGS_LOCK();
}

/**
  * @brief open or close CSM modules
  * @param OnOffState   open or close related clock gate
  *     @arg @b  SH_ON  open related clock gate
  *     @arg @b  SH_OFF close related clock gate
  * @retval   None
  **/
void RCC_CSMModuleOnOff(CmdState OnOffState)
{
    assert_param(IS_CMD_STATE(OnOffState));

    /* Unlock RCC registers */
    RCC_REGS_UNLOCK();

    RCC->CR.BIT.CSMON = OnOffState;

    /* Lock RCC registers */
    RCC_REGS_LOCK();
}

/**
  * @brief     Get RCC CSM flag
  * @param  ResetFlag:   RCC CSM flag mask
  *     @arg @b  RCC_CSM_HSE_RF
  *     @arg @b  RCC_CSM_PLL_RF
  *     @arg @b  RCC_CSM_HSE_IF
  *     @arg @b  RCC_CSM_PLL_IF
  * @retval  FlagStatus RCC CSM flag
  *     @arg @b  SH_SH_SET the flag is setted
  *     @arg @b  SH_RESET  the flag is cleared
  **/
FlagStatus RCC_GetCsmFStatus(RCC_CSMF_Type CSMFlag)
{
    assert_param(IS_CSM_FLAG(CSMFlag));

    return ((RCC->CISTR.V32 & CSMFlag) ? SH_SET : SH_RESET);
}

/**
  * @brief Clear clock inerrupt flag
  * @param FlagStatus RCC CSM flag
  *     @arg @b  RCC_CSM_CICCLR_CSMC       Clock security monitor interrupt clear
  *     @arg @b  RCC_CSM_CICCLR_PLLRDYC    PLL ready interrupt clear
  *     @arg @b  RCC_CSM_CICCLR_HSERDYC    HSE ready interrupt clear
  * @retval  null
  **/
void RCC_ClearITPendingBit(RCC_CICCLR_Type CLK_INTFLG_CLR)
{
    assert_param(IS_CICCLR_FLAG(CLK_INTFLG_CLR));
    /* Unlock RCC registers */
    RCC_REGS_UNLOCK();
    if (CLK_INTFLG_CLR == RCC_CSM_CICCLR_CSMC)
    {
        RCC->CICLR.BIT.CSMC = 1;
    }
    else if (CLK_INTFLG_CLR == RCC_CSM_CICCLR_PLLRDYC)
    {
        RCC->CICLR.BIT.PLLRDYC = 1;
    }
    else
    {
        RCC->CICLR.BIT.HSERDYC = 1;
    }
    /* Unlock RCC registers */
    RCC_REGS_LOCK();
}

/**
  * @brief Enable clock inerrupt
  * @param  RCC_CIENR_Type   PLL/HSE ready inerrupt enable
  *     @arg @b  RCC_CIENR_HSERDYIE    HSE ready inerrupt enable
  *     @arg @b  RCC_CIENR_PLLRDYIE    PLL  ready inerrupt enable
  * @param  FunctionalState     PLL/HSE enable status
  *     @arg @b  SH_DISABLE      disable inetrrupt
  *     @arg @b  SH_ENABLE       enable interrupt
  * @retval null
  **/
void RCC_INTConfig(RCC_CIENR_Type RDYINTIE, FunctionalState NewState)
{
    assert_param(IS_CIENR_REGISTER(RDYINTIE));
    assert_param(IS_FUNCTION_STATE(NewState));

    RCC_REGS_UNLOCK();
    if (NewState != SH_DISABLE)
    {
        RCC->CIENR.V32 |= RDYINTIE;
    }
    else
    {
        RCC->CIENR.V32 &= (~RDYINTIE);
    }
    RCC_REGS_LOCK();
}

/**
  * @}
  */

/**
  * @}
  */

/************************ (C) COPYRIGHT Sinowealth *****END OF FILE****/
